from RPLCD import CharLCD
import os
import sys
import subprocess
from RPi import GPIO
from time import sleep
from Receiver import Receiver
from RotarySwitch import RotarySwitch
from Switch import Switch
from Potentiometer import Potentiometer

#########################################################################################
# GPIO pins allocation (pins number not GPIO number)
#
# LCD Screen
PIN_LCD_RS = 36
PIN_LCD_E = 35
PIN_LCD_DATA1 = 33
PIN_LCD_DATA2 = 31
PIN_LCD_DATA3 = 29
PIN_LCD_DATA4 = 32
#
# Rotary switch for station tuning
PIN_STA_CLK = 12   # GPIO 18
PIN_STA_DT =  11   # GPIO 17
#
# Power ON/OFF switch
PIN_POWER_SWITCH = 8
#
# MCP3008 analog to digital converter for volume potentiometer
PIN_MCP_CLK = 23        # CLK GPIO 11
PIN_MCP_MOSI = 19       # MOSI GPIO 10
PIN_MCP_MISO = 21       # MISO GPIO 9
PIN_MCP_CSPI = 24       # CS0 GPIO 8
#########################################################################################
# MCP3008 channel used for potentiometer
MCP_ADCNUM = 0   
#########################################################################################


class WebRadio:

    def __init__(self, StationListFile):
    
        # Set GPIO mode to use the physical pin numbers on the GPIO connector
        GPIO.setmode(GPIO.BOARD)
    
        # LCD declaration
        self.lcd = CharLCD(cols=16, rows=2, pin_rs=PIN_LCD_RS, pin_e=PIN_LCD_E, pins_data=[PIN_LCD_DATA1, PIN_LCD_DATA2, PIN_LCD_DATA3, PIN_LCD_DATA4],numbering_mode=GPIO.BOARD)
        self.lcd.clear()
        
        # Create Receiver object and initialize the stations
        self.receiver = Receiver(StationListFile)
        
        # Initialize the tune button (change station)
        self.tuneBtn = RotarySwitch(PIN_STA_CLK, PIN_STA_DT, self.tune)
        
        # Initialize the Power on/off button
        self.powerBtn = Switch(PIN_POWER_SWITCH, self.power)
        
        # Initialize the Volume button
        self.volumeBtn = Potentiometer(MCP_ADCNUM, PIN_MCP_CLK, PIN_MCP_MOSI, PIN_MCP_MISO, PIN_MCP_CSPI, self.volume)
        
        # Start if power is on
        if self.powerBtn.getStatus() == Switch.ON:
            print("Power button is ON")
            self.start()
        else:
            self.lcd.clear()
            self.lcd.write_string("Tournez le \n\rbouton volume")


    def power(self, status):
        #
        # Handle power ON and OFF events
        #
        if status == Switch.ON:
            self.start()
        else:
            self.shutdown()

    def start(self):
        #
        # Start the radio, play the last station listened
        #
        self.volumeBtn.start()
        self.play()

    def shutdown(self):
        #
        # Shutdown the system (to do before switching power off)
        #
        self.volumeBtn.stop()
        self.receiver.stop()
        self.lcd.clear()
        self.lcd.write_string(u'Bye-bye...')
        GPIO.cleanup()
        # System shutdown
        os.system("sudo shutdown -h now")
        

    def play(self):
        #
        # Play the current station memorized
        # 
        caption = self.receiver.play()
        # display station name and volume
        print("Play station : " + caption)
        self.display()
        

    def tune(self, clockwise):
        #
        # Called when tune button is turned to change the web station
        #
        if clockwise:
            caption = self.receiver.playNext()
        else:
            caption = self.receiver.playPrevious()
        # display station name and volume 
        print("Changed station to :" + caption) 
        self.display()
    
    def display(self):
        #  
        # Display the radio caption and volume
        #
        # Stop any interruption from potentiometer to preserve display integrity
        self.volumeBtn.stop()
        
        # display station name on the 1st line
        self.lcd.clear()
        self.lcd.cursor_pos = (0,0)
        trackName = '{:16.16}'.format(self.receiver.getStationName())
        self.lcd.write_string(trackName) 
        # display volume on 2nd line
        self.lcd.cursor_pos = (1,0)
        volumeLevel = '{:16.16}'.format("Volume: " + str(self.receiver.getVolume()) + "%")
        self.lcd.write_string(volumeLevel)  
        
        # Restart potentiometer
        self.volumeBtn.start() 
        
    def volume(self, value):
        #
        # Called when volume button is turned
        #
        self.receiver.setVolume(value) 
        self.display()         

###################
#
# Main
#
if __name__ == "__main__":
    import sys
    sys_path = os.path.abspath(sys.path[0])
    
    # Create web radio object 
    webradio = WebRadio("Stations.xml")
    
    # Wait for ever
    while True:
        sleep(10)

